﻿/*
 * Created by SharpDevelop.
 * User: jt
 * Date: 30-10-2010
 * Time: 23:07
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;

namespace HovText
{
	/// <summary>
	/// Description of ClipboardHistory.
	/// </summary>
	public class ClipboardHistory
	{
		
		class HistoryItem 
		{
			Dictionary<string, object> dict = new Dictionary<string, object> ();
			IDataObject ido;
			Bitmap bmp;
			
			public HistoryItem(IDataObject obj)
			{
				DbgLog.Instance.Log ("ClipboardHistory.Item()");
			 	if (!(obj.GetDataPresent(DataFormats.Bitmap) 
				      ||obj.GetDataPresent(DataFormats.StringFormat) ))
				    return;

				ido = obj;
				bool saveBmp;
				string[] fmts = FormatsToSave(obj, out saveBmp);
				foreach (string f in fmts)
				{
					DbgLog.Instance.Log ("GetData: " + f);						
					object data = obj.GetData(f);
					DbgLog.Instance.Log ("OK");
					if (data != null)
					{
					    dict.Add(f, data);
						if (f.Equals("System.String"))
							DbgLog.Instance.Log("Found string value: \"" + (string) data +"\"");
						DbgLog.Instance.Log ("Added to dict");
					}
				}
			 	if(saveBmp && obj.GetDataPresent(DataFormats.Bitmap))
			 	{
			 		DbgLog.Instance.Log ("Get BMP");
    			    bmp =  (Bitmap)obj.GetData(DataFormats.Bitmap);
			 		DbgLog.Instance.Log ("OK");
			 	}
    			DbgLog.Instance.Log ("Item() - done\r\n\r\n");
			}
			
			
			string[] FormatsToSave(IDataObject obj, out bool SaveBmp)
			{
				List<string> result = new List<string>();
				string[] fmts = obj.GetFormats();
				bool isSpreadSheet = false;
				SaveBmp = true;
				foreach (string f in fmts)
				{
					if (f.Equals("XML Spreadsheet")||f.Equals("Wk1"))
						isSpreadSheet = true;
				}
				foreach (string f in fmts)
				{
					bool ok = false;
					if (f.Equals("HTML Format")
						    || f.Equals("System.String")
						    || f.ToUpper().StartsWith("UnicodeText")
						    || f.Equals("Text")
						    || f.Equals("Rich Text Format")
						    || f.StartsWith("Bitmap"))
						ok=true;
					if (ok && isSpreadSheet)
					{
						SaveBmp =false;
						ok = false;
						if (f.Equals("System.String") 
						      // || f.Equals("Rich Text Format")
						   )
							ok = true;
					}
					if (ok)
						result.Add(f);
				}
				return result.ToArray();
			}
			public object Summary()
			{
				if (dict.ContainsKey("System.String") && dict["System.String"] != null)
					return (string) dict["System.String"];
				if (bmp != null)
					return bmp;
				return "";
			}
			
			public override string ToString()
			{
				return Summary().ToString();
			}
			
			public bool HasValue()
			{
				if (dict.ContainsKey("System.String") && dict["System.String"] != null)
					return ((string) dict["System.String"]).Length >0;
				return (bmp != null);
			}
			
			public void Restore()
			{
				DataObject data = new DataObject();
				foreach (KeyValuePair<string, object> kvp in dict)
				{
					if (kvp.Value != null)
						data.SetData(kvp.Key, kvp.Value);
				}
				Clipboard.SetDataObject(data,true);					
			}
			
			public override int GetHashCode()
			{
				return dict.GetHashCode();
			}
			
			public override bool Equals(object obj)
			{
				HistoryItem item = obj as HistoryItem;
				if (item != null)
				{
					if (dict.ContainsKey("System.String") &&
					    item.dict.ContainsKey("System.String"))
					{
						if (dict["System.String"] == null)
						{
							return (item.dict["System.String"] == null);
						} else {
							if (item.dict["System.String"] == null)
								return false;
						}
						return ((string) dict["System.String"]).Trim().Equals(((string) item.dict["System.String"]).Trim());
					}
					if (bmp != null && item.bmp != null)
						return bmp.Equals(item.bmp);
				}
				return base.Equals(obj);
			}
		}

		List<HistoryItem> items = new List<HistoryItem>();

		
		public ClipboardHistory()
		{
			
		}
		
		int AlreadyInsertedAt(HistoryItem test)
		{
			for (int i = 0; i< items.Count; i++)
				if (items[i].Equals(test))
				    return i;
			return -1;
		}
		
		public bool Add(IDataObject obj)
		{
			HistoryItem newitem = new HistoryItem(obj);
			if (newitem.HasValue())
			{
				Add(newitem);
				return true;
			}
			return false;
		}
		
		private void Add(HistoryItem newitem)
		{
			int oldpos = AlreadyInsertedAt(newitem);
			if (oldpos == -1)
			{
				DbgLog.Instance.Log("History added new item: " + newitem.ToString());
				items.Insert(0, newitem );
				while (items.Count > Settings.Instance.HistorySize)
					items.RemoveAt(items.Count - 1);
			} else {
				DbgLog.Instance.Log("History replaced item: " + newitem.ToString());
				items.RemoveAt(oldpos);
				items.Insert(0, newitem ); 
			}
		}
		
		public object GetSummary(int i)
		{
			return items[i].Summary();
		}

		public void Restore(int i)
		{
			Add(items[i]);
			items[0].Restore();
		}
		
		public int Count {
			get 
			{ 
				while (items.Count > Settings.Instance.HistorySize)
					items.RemoveAt(items.Count - 1);
				return items.Count; 
			}
		}
		
	}
}
