﻿/*
 * Created by SharpDevelop.
 * User: jt
 * Date: 23-10-2010
 * Time: 19:02
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.IO;
using System.Net;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Resources;
using System.Reflection;
using System.Runtime.InteropServices;

namespace HovText
{

	public sealed class NotificationIcon
	{
		private NotifyIcon notifyIcon;
		private ClipboardMonitor clipMon = null;
		private ClipboardManipulator clipMan =null;
		private ClipboardHistory clipHist = null;
		private ResourceManager resources;
		private HotkeyFilter hotkey, histHotkey, histBackHotKey;
		private bool Active = true;
		
		#region Initialize icon and menu
		public NotificationIcon()
		{
			notifyIcon = new NotifyIcon();
			
			notifyIcon.MouseClick += new MouseEventHandler(notifyIcon_MouseClick);
			notifyIcon.MouseDoubleClick += new MouseEventHandler(notifyIcon_MouseDoubleClick);

			resources = new ResourceManager("HovText.Resource", Assembly.GetExecutingAssembly());
			
			Active = true;
			Init();
			if (Settings.Instance.IsFirstRun())
			{
				new WelcomeForm().ShowDialog();
				Settings.Instance.Save();
				ShowSettings();
			}
			else if (Settings.Instance.CheckForUpdates)
				DoCheckUpdates();
		}

		
	
		private MenuItem[] InitializeMenu()
		{
			MenuItem settings= new MenuItem("Settings".GT(), menuSettingsClick);
			settings.DefaultItem = true;
			MenuItem[] menu = new MenuItem[] {
				new MenuItem("About".GT(), menuAboutClick),
				new MenuItem("Feedback".GT(), menuFeedbackClick),
				settings,
				new MenuItem("Exit".GT(), menuExitClick)
			};
			return menu;
		}
		#endregion
		
		
		void Init()
		{			
			if (clipMon != null)
			{
				clipMon.Dispose();
				clipMon = null;
			}
			if (clipMan != null)
			{
				clipMan = null;
			}
			clipMon = new ClipboardMonitor();
			clipMon.ClipboardChanged += new EventHandler<ClipboardChangedEventArgs>(clipMon_ClipboardChanged);
			clipMan = new ClipboardManipulator();
			if (clipHist == null)
				clipHist = new ClipboardHistory();
			
			Translation.Instance.SetLang(Settings.Instance.LangCode);
			
			SetHotKeys();
			notifyIcon.ContextMenu = new ContextMenu(InitializeMenu());
			if (Settings.Instance.MainHKFunction == HKFUNCTION.REMOVE_FMT)
			{
				notifyIcon.Icon = GetIcon("hotkey");
				Active = false;
			}
			else if (Active)
				notifyIcon.Icon = GetIcon("active");
			else
				notifyIcon.Icon = GetIcon("inactive");
		}
		
		
		void SetHotKeys()
		{
			if (hotkey !=null)
			{
				hotkey.Unregister();
				hotkey = null;
			}
			if (Settings.Instance.MainHotKey.KeyCode != Keys.None)
			{
				hotkey = new HotkeyFilter(Settings.Instance.MainHotKey);
				hotkey.Pressed += new HandledEventHandler(hotkey_Pressed);
				hotkey.Register(clipMon);
			}
			
			if (histHotkey !=null)
			{
				histHotkey.Unregister();
				histHotkey = null;
			}
			if (histBackHotKey != null)
			{
				histBackHotKey.Unregister();
				histBackHotKey = null;
			}
			if (Settings.Instance.HistoryHotKey.KeyCode != Keys.None)
			{
				histHotkey = new HotkeyFilter(Settings.Instance.HistoryHotKey);
				histHotkey.Pressed += new HandledEventHandler(histHotkey_Pressed);
				histHotkey.Register(clipMon);
				if (!Settings.Instance.HistoryHotKey.Shift)
				{
					HotKey hk = new HotKey()
					{
						KeyCode=Settings.Instance.HistoryHotKey.KeyCode,
						Alt = Settings.Instance.HistoryHotKey.Alt,
						Control = Settings.Instance.HistoryHotKey.Control,
						Windows = Settings.Instance.HistoryHotKey.Windows,
						Shift = true,
					};
					histBackHotKey = new HotkeyFilter(hk);
					histBackHotKey.Pressed += new HandledEventHandler(histBackHotkey_Pressed);
					histBackHotKey.Register(clipMon);
					
				}
			}
		}
		
		Icon GetIcon(string name)
		{
			if (File.Exists(name + ".ico"))
			   return  Icon.ExtractAssociatedIcon(name + ".ico");
			return (Icon)resources.GetObject(name);
		}
		void histHotkey_Pressed(object sender, HandledEventArgs e)
		{
			ShowHistory(true);
		}
		void histBackHotkey_Pressed(object sender, HandledEventArgs e)
		{
			ShowHistory(false);
		}

		int showNo = 0;
		void ShowHistory(bool forward)
		{
			if (clipHist != null && clipHist.Count>0)
			{
				if (ShowHistoryItemForm.IsVisible())
					showNo += forward ? 1: -1;
				else
					showNo = 0;
				if (showNo >= clipHist.Count)
					showNo = 0;
				if (showNo < 0)
					showNo = clipHist.Count-1;
				bool last =(showNo==clipHist.Count-1);
				object summary = clipHist.GetSummary(showNo);
				if ((Active || Settings.Instance.MainHKFunction == HKFUNCTION.REMOVE_FMT)
				    && summary is string)
					summary = clipMan.ProcessString((string)summary);
				ShowHistoryItemForm.ShowBox(summary, HistoryClosed , last ? Color.Red : Color.Black);
			}
		}
		
		void HistoryClosed()
		{
			historymanipulation = true;
			clipHist.Restore(showNo);
			Application.DoEvents();
			historymanipulation = false;
		}		

		void hotkey_Pressed(object sender, HandledEventArgs e)
		{
			switch (Settings.Instance.MainHKFunction )
			{
				case HKFUNCTION.ACTIVATE_DEACTIVATE:
					ToggleActive();
					break;
				case HKFUNCTION.REMOVE_FMT:
					
					IDataObject obj =Clipboard.GetDataObject();
					HistoryItem hi = new HistoryItem(obj);
					ProcessClipboard(obj);
					SendKeys.Send("^v");
					System.Windows.Forms.Timer tm;
					tm = new System.Windows.Forms.Timer();
					tm.Interval = 500;
					tm.Tick += 
						delegate 
					{
						tm.Enabled=false; 
						tm = null; 
						hi.Restore(); 
					};
					tm.Enabled=true;
					break;
			}
		}
		
		
		void ToggleActive()
		{
			Active = !Active;
			historymanipulation = true;
			if (Active)
			{ 
				ProcessClipboard(Clipboard.GetDataObject());
				notifyIcon.Icon = GetIcon("active");
			} else {
				if (clipHist.Count >0)
			    	clipHist.Restore(0);
				notifyIcon.Icon = GetIcon("inactive");
			}
			Application.DoEvents();
			historymanipulation = false;
		}

		bool manipulation = false;
		bool historymanipulation = false;
		void clipMon_ClipboardChanged(object sender, ClipboardChangedEventArgs e)
		{
			DbgLog.Instance.Log ("clipMon_ClipboardChanged");
			if (Settings.Instance.HistoryHotKey.KeyCode != Keys.None && 
			    !(historymanipulation || manipulation))
			{
				DbgLog.Instance.Log ("clipHist.Add");
				clipHist.Add(e.DataObject);
			}
			if (Active)
				ProcessClipboard(e.DataObject);
			DbgLog.Instance.Log ("clipMon_ClipboardChanged - done");
		}
		
		void ProcessClipboard(IDataObject DataObject)
		{
			DbgLog.Instance.Log ("ProcessClipboard");
			if (manipulation)
				return;
			manipulation = true;
			if (clipMan != null)
				clipMan.Process(DataObject);
			Application.DoEvents();
			DbgLog.Instance.Log ("ProcessClipboard - done");
			Application.DoEvents();
			manipulation = false;
		}
		
		public void Stop()
		{
			if (clipMon != null)
				clipMon.Stop();
		}
			
			
		private void ShowSettings()
		{
			if (sf != null)
			{
				SetForegroundWindow(sf.Handle);
				sf.Focus();
			} else
				{
					if (hotkey !=null)
					{
						hotkey.Unregister();
						hotkey = null;
					}
					if (histHotkey !=null)
					{
						histHotkey.Unregister();
						histHotkey = null;
					}
					HKFUNCTION  lastHKFunc = Settings.Instance.MainHKFunction;
					sf = new SetupForm();
					if (sf.ShowDialog() == DialogResult.OK)
					{
						if (lastHKFunc == HKFUNCTION.REMOVE_FMT
						    && Settings.Instance.MainHKFunction == HKFUNCTION.ACTIVATE_DEACTIVATE)
							Active = true;
						this.Init();
						Settings.Instance.Save();
					} else
						SetHotKeys();
					sf = null;
				}
		}
		
		void Close()
		{
			Application.Exit();
		}
		
		#region Main - Program entry point
		/// <summary>Program entry point.</summary>
		/// <param name="args">Command Line Arguments</param>
		[STAThread]
		public static void Main(string[] args)
		{
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
						
			bool isFirstInstance;
			// Please use a unique name for the mutex to prevent conflicts with other programs
			using (Mutex mtx = new Mutex(true, "HovText", out isFirstInstance)) {
				if (isFirstInstance) {
					foreach (string arg in args)
					{
						if (arg.Equals("/debug"))
						{
							DbgLog.Instance.Active = true;
						}
					}
					NotificationIcon notificationIcon = new NotificationIcon();
					notificationIcon.notifyIcon.Visible = true;
					Application.Run();
					notificationIcon.Stop();
					notificationIcon.notifyIcon.Dispose();
				} else {
					// The application is already running
				}
			} // releases the Mutex
		}
		#endregion
		
		#region Event Handlers
		private void menuSettingsClick(object sender, EventArgs e)
		{
			ShowSettings();
		}
		
		private void menuAboutClick(object sender, EventArgs e)
		{
			new AboutForm().ShowDialog();
		}
		
		private void menuFeedbackClick(object sender, EventArgs e)
		{ 
			new FeedbackForm().ShowDialog();
		}
		
		private void menuExitClick(object sender, EventArgs e)
		{
			Close();
		}
		
		
		[DllImport("user32.dll")]
		public static extern bool SetForegroundWindow(IntPtr hWnd);
		

		DateTime clickTime;
		void notifyIcon_MouseClick(object sender, MouseEventArgs e)
		{
			if (Settings.Instance.MainHKFunction == HKFUNCTION.ACTIVATE_DEACTIVATE
			   && e.Button == MouseButtons.Left)		
			{
				if (DateTime.Now.Subtract(clickTime).TotalMilliseconds>500)
				{
					clickTime = DateTime.Now;
					ToggleActive();
				}
			}
		}
		
		
		SetupForm sf = null;
		void notifyIcon_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			if (Settings.Instance.MainHKFunction == HKFUNCTION.ACTIVATE_DEACTIVATE
			   && e.Button == MouseButtons.Left)
			{
				if (DateTime.Now.Subtract(clickTime).TotalMilliseconds<500)
				{
					ToggleActive();
				}
				clickTime = DateTime.Now;
			}
			ShowSettings();
		}
		#endregion
		
		#region Updates
		
		private void DoCheckUpdates()
		{
			bool DoChk=false;
			if (VersionInfo.Instance.IsAlpha())
			    DoChk = true;
			else
				DoChk = (DateTime.Now.Subtract(Settings.Instance.UpdateReminded).Days>=14);
			if (DoChk)
			{
			new Thread (ChkThread).Start();
			tm = new System.Threading.Timer(ChkReply,null,500,100);
			}
		}
		System.Threading.Timer tm;
		
		private void ChkReply(object obj)
		{
			if (!string.IsNullOrEmpty(serverVersion))
			{
				serverVersion = serverVersion.Trim();
				tm.Change(Timeout.Infinite, Timeout.Infinite);
				string thisVer = VersionInfo.Instance.GetString();
				if (serverVersion.StartsWith("Version:") && !thisVer.Equals(serverVersion.Substring(9).Trim()))
				{
	    			if (DateTime.Now.Subtract(Settings.Instance.UpdateReminded).Days>=14)
					{
						new UpdateForm(serverVersion.Substring(9).Trim()).ShowDialog();
						Settings.Instance.UpdateReminded = DateTime.Now;
						Settings.Instance.Save();
					}
				}
			}
		}
		
				
		string serverVersion = null;
		private void ChkThread()
		{
			try
			{
				HttpWebRequest req = (HttpWebRequest)WebRequest.Create("http://hovtext.com/autoupdate/");

				req.UserAgent = "HovText " + VersionInfo.Instance.GetString();
				HttpWebResponse reply = (HttpWebResponse)req.GetResponse();
			    serverVersion = new StreamReader(reply.GetResponseStream()).ReadToEnd();			
			} catch 
			{
			}
		}

		#endregion
	}
}
